@@ -12,31 +12,41 @@
 #' @param ... Optionally named data frames.
 #' @param names A character vector describing provided tables names.
 #'   If missing names are constructed based on provided tables objects.
+#' @param .class The extra (highest priority) class added to the resulting object.
+#'   Having the extra class defined, enables to implement custom S3 methods for the object
+#'   having higher priority over the existing methods.
+#'   Especially useful if you want to change the built-in method behavior.
 #' @return Object of class 'tblist' being a named list of data frames.
 #' @export
-tblist <- function(..., names) {
+tblist <- function(..., names, .class = NULL) {
   tables <- rlang::dots_list(..., .named = TRUE)
-  out_class <- "tblist"
+  out_class <- c(.class, "tblist")
 
-  tb_call <- sys.call(1)
-  if (inherits(tables, "data.frame")) {
-    if (!missing(names)) {
-      names(tables) <- names
-    }
-    return(
-      structure(tables, class = out_class)
-    )
-  }
-  if (inherits(tables, "list")) {
+  tb_call <- sys.call(1L)
+  if (purrr::every(tables, is.data.frame)) {
     if (!missing(names)) {
       if (length(tables) != length(names)) {
         stop(glue::glue(
           "{sQuote('tables')} should be of same length as {sQuote('names')}"
         ))
       }
+      names(tables) <- names
+    }
+    return(
+      structure(tables, class = out_class)
+    )
+  }
+
+  if (inherits(tables[[1L]], "list") && length(tables) == 1L) {
+    if (!missing(names)) {
+      if (length(tables[[1L]]) != length(names)) {
+        stop(glue::glue(
+          "{sQuote('tables')} should be of same length as {sQuote('names')}"
+        ))
+      }
       return(
         structure(
-          stats::setNames(tables, names),
+          stats::setNames(tables[[1L]], names),
           class = out_class
         )
       )
@@ -53,18 +63,18 @@ tblist <- function(..., names) {
 #' @param x an R object.
 #' @param ... additional arguments to be passed to or from methods.
 #' @export
-as.tblist <- function(x, ...) {
+as.tblist <- function(x, ..., .class = NULL) {
   UseMethod("as.tblist", x)
 }
 
 #' @export
-as.tblist.data.frame <- function(x, names, ...) {
-  tblist(x, names = names)
+as.tblist.data.frame <- function(x, names, ..., .class = NULL) {
+  tblist(x, names = names, .class = .class)
 }
 
 #' @export
-as.tblist.list <- function(x, names, ...) {
-  tblist(!!!x, names = names)
+as.tblist.list <- function(x, names, ..., .class = NULL) {
+  tblist(!!!x, names = names, .class = .class)
 }
 
 #' @rdname set_source
@@ -109,8 +119,8 @@ set_source.tblist <- function(dtconn, primary_keys = NULL, binding_keys = NULL,
 #' @param keep_na If `TRUE`, NA values are included.
 #' @export
 cb_filter.discrete.tblist <- function(
-  source, type = "discrete", id = .gen_id(), name = id, variable, value = NA,
-  dataset, keep_na = TRUE, ..., description = NULL, active = TRUE) {
+    source, type = "discrete", id = .gen_id(), name = id, variable, value = NA,
+    dataset, keep_na = TRUE, ..., description = NULL, active = TRUE) {
   args <- list(...)
 
   def_filter(
@@ -146,22 +156,23 @@ cb_filter.discrete.tblist <- function(
       }
       stats <- list(
         choices = if ("choices" %in% name) data_object[[dataset]][[variable]] %>%
-          stats::na.omit() %>% table() %>% as.list(),
-        n_data = if ("n_data" %in% name)  data_object[[dataset]][[variable]] %>%
-          stats::na.omit() %>% length(),
+          stats::na.omit() %>%
+          table() %>%
+          as.list(),
+        n_data = if ("n_data" %in% name)  data_object[[dataset]][[variable]] %>% stats::na.omit() %>% length(),
         n_missing = if ("n_missing" %in% name) data_object[[dataset]][[variable]] %>% is.na() %>% sum()
       )
-      if (length(name) == 1) {
+      if (length(name) == 1L) {
         return(stats[[name]])
       } else {
         return(stats[name])
       }
     },
-    plot_data = function(data_object) {
+    plot_data = function(data_object, ...) {
       if (nrow(data_object[[dataset]])) {
-        data_object[[dataset]][[variable]] %>% table %>% prop.table() %>% graphics::barplot()
+        data_object[[dataset]][[variable]] %>% table %>% prop.table() %>% graphics::barplot(...)
       } else {
-        graphics::barplot(0, ylim = c(0, 0.1), main = "No data")
+        graphics::barplot(0.0, ylim = c(0.0, 0.1), main = "No data")
       }
     },
     get_params = function(name) {
@@ -189,8 +200,8 @@ cb_filter.discrete.tblist <- function(
 #' @rdname filter-source-types
 #' @export
 cb_filter.discrete_text.tblist <- function(
-  source, type = "discrete_text", id = .gen_id(), name = id, variable, value = NA,
-  dataset, ..., description = NULL, active = TRUE) {
+    source, type = "discrete_text", id = .gen_id(), name = id, variable, value = NA,
+    dataset, ..., description = NULL, active = TRUE) {
   args <- list(...)
 
   def_filter(
@@ -207,7 +218,7 @@ cb_filter.discrete_text.tblist <- function(
             !!sym(variable) %in% !!strsplit(
               sub(" ", "", value, fixed = TRUE),
               split = ",", fixed = TRUE
-            )[[1]]
+            )[[1L]]
           )
         # keep_na !value_na end, # !keep_na !value_na end
       }
@@ -219,17 +230,22 @@ cb_filter.discrete_text.tblist <- function(
         name <- c("n_data", "choices", "n_missing")
       }
       stats <- list(
-        choices = if ("choices" %in% name) data_object[[dataset]][[variable]] %>% unique() %>% paste(collapse = ","),
-        n_data = if ("n_data" %in% name)  data_object[[dataset]][[variable]] %>% stats::na.omit() %>% unique() %>% length(),
+        choices = if ("choices" %in% name) data_object[[dataset]][[variable]] %>%
+          collapse::funique() %>%
+          paste(collapse = ","),
+        n_data = if ("n_data" %in% name)  data_object[[dataset]][[variable]] %>%
+          stats::na.omit() %>%
+          collapse::funique() %>%
+          length(),
         n_missing = if ("n_missing" %in% name) data_object[[dataset]][[variable]] %>% is.na() %>% sum()
       )
-      if (length(name) == 1) {
+      if (length(name) == 1L) {
         return(stats[[name]])
       } else {
         return(stats[name])
       }
     },
-    plot_data = function(data_object) {},
+    plot_data = function(data_object, ...) {},
     get_params = function(name) {
       params <- list(
         dataset = dataset,
@@ -252,14 +268,15 @@ cb_filter.discrete_text.tblist <- function(
 }
 
 get_range_frequencies <- function(data_object, dataset, variable, extra_params) {
-  step <- 1
-  if (length(stats::na.omit(data_object[[dataset]][[variable]])) == 0) {
+  step <- 1L
+  if (length(stats::na.omit(data_object[[dataset]][[variable]])) == 0L) {
     return(
       data.frame(
-        level = character(0),
-        count = numeric(0),
-        l_bound = numeric(0),
-        u_bound = numeric(0)
+        level = character(0L),
+        count = numeric(0L),
+        l_bound = numeric(0L),
+        u_bound = numeric(0L),
+        stringsAsFactors = FALSE
       )
     )
   }
@@ -271,7 +288,8 @@ get_range_frequencies <- function(data_object, dataset, variable, extra_params)
         level = "1",
         count = length(data_object[[dataset]][[variable]]),
         l_bound = min_val,
-        u_bound = max_val
+        u_bound = max_val,
+        stringsAsFactors = FALSE
       )
     )
   }
@@ -280,13 +298,13 @@ get_range_frequencies <- function(data_object, dataset, variable, extra_params)
     step <- extra_params$step
   }
   breaks <- seq(min_val, max_val, by = step)
-  if (rev(breaks)[1] != max_val) {
-    breaks[length(breaks) + 1]  <- max_val
+  if (rev(breaks)[1L] != max_val) {
+    breaks[length(breaks) + 1L]  <- max_val
   }
-  breaks <- round(breaks, 2)
+  breaks <- round(breaks, 2L)
   bounds <- breaks
 
-  breaks[1] <- breaks[1] - 0.01
+  breaks[1L] <- breaks[1L] - 0.01
   breaks[length(breaks)] <- breaks[length(breaks)] + 0.01
 
   data_object[[dataset]][, variable, drop = FALSE] %>%
@@ -294,19 +312,19 @@ get_range_frequencies <- function(data_object, dataset, variable, extra_params)
     dplyr::mutate(
       level = factor(
         findInterval(!!sym(variable), breaks, rightmost.closed = FALSE),
-        levels = 1:(length(breaks)),
-        labels = as.character(1:(length(breaks)))
+        levels = seq_along(breaks),
+        labels = as.character(seq_along(breaks))
       )
     ) %>%
     dplyr::group_by(level) %>%
     dplyr::summarise(
       count = dplyr::n()
     ) %>%
-    tidyr::complete(level, fill = list(count = 0)) %>%
+    tidyr::complete(level, fill = list(count = 0L)) %>%
     dplyr::arrange(level) %>%
     dplyr::mutate(
       l_bound = bounds,
-      u_bound = c(bounds[-1], bounds[length(bounds)])
+      u_bound = c(bounds[-1L], bounds[length(bounds)])
     )
 }
 
@@ -314,8 +332,8 @@ get_range_frequencies <- function(data_object, dataset, variable, extra_params)
 #' @param range Variable range to be applied in filtering.
 #' @export
 cb_filter.range.tblist <- function(
-  source, type = "range", id = .gen_id(), name = id, variable, range = NA, dataset,
-  keep_na = TRUE, ..., description = NULL, active = TRUE) {
+    source, type = "range", id = .gen_id(), name = id, variable, range = NA, dataset,
+    keep_na = TRUE, ..., description = NULL, active = TRUE) {
   args <- list(...)
 
   def_filter(
@@ -328,7 +346,7 @@ cb_filter.range.tblist <- function(
       if (keep_na && !identical(range, NA)) {
         # keep_na !value_na start
         data_object[[dataset]] <- data_object[[dataset]] %>%
-          dplyr::filter((!!sym(variable) <= !!range[2] & !!sym(variable) >= !!range[1]) | is.na(!!sym(variable)))
+          dplyr::filter((!!sym(variable) <= !!range[2L] & !!sym(variable) >= !!range[1L]) | is.na(!!sym(variable)))
         # keep_na !value_na end
       }
       if (!keep_na && identical(range, NA)) {
@@ -340,7 +358,7 @@ cb_filter.range.tblist <- function(
       if (!keep_na && !identical(range, NA)) {
         # !keep_na !value_na start
         data_object[[dataset]] <- data_object[[dataset]] %>%
-          dplyr::filter(!!sym(variable) <= !!range[2] & !!sym(variable) >= !!range[1])
+          dplyr::filter(!!sym(variable) <= !!range[2L] & !!sym(variable) >= !!range[1L])
         # !keep_na !value_na end
       }
       attr(data_object[[dataset]], "filtered") <- TRUE # code include
@@ -359,17 +377,17 @@ cb_filter.range.tblist <- function(
         n_data = if ("n_data" %in% name)  data_object[[dataset]][[variable]] %>% stats::na.omit() %>% length(),
         n_missing = if ("n_missing" %in% name) data_object[[dataset]][[variable]] %>% is.na() %>% sum()
       )
-      if (length(name) == 1) {
+      if (length(name) == 1L) {
         return(stats[[name]])
       } else {
         return(stats[name])
       }
     },
-    plot_data = function(data_object) {
+    plot_data = function(data_object, ...) {
       if (nrow(data_object[[dataset]])) {
-        data_object[[dataset]][[variable]] %>% graphics::hist()
+        data_object[[dataset]][[variable]] %>% graphics::hist(...)
       } else {
-        graphics::barplot(0, ylim = c(0, 0.1), main = "No data")
+        graphics::barplot(0.0, ylim = c(0.0, 0.1), main = "No data")
       }
     },
     get_params = function(name) {
@@ -391,8 +409,8 @@ cb_filter.range.tblist <- function(
     get_defaults = function(data_object, cache_object) {
       list(
         range = c(
-          cache_object$frequencies$l_bound[1],
-          rev(cache_object$frequencies$u_bound)[1]
+          cache_object$frequencies$l_bound[1L],
+          rev(cache_object$frequencies$u_bound)[1L]
         )
       )
     }
@@ -401,13 +419,14 @@ cb_filter.range.tblist <- function(
 
 get_date_range_frequencies <- function(data_object, dataset, variable, extra_params) {
   step <- "day"
-  if (length(stats::na.omit(data_object[[dataset]][[variable]])) == 0) {
+  if (length(stats::na.omit(data_object[[dataset]][[variable]])) == 0L) {
     return(
       data.frame(
-        level = character(0),
-        count = numeric(0),
-        l_bound = numeric(0),
-        u_bound = numeric(0)
+        level = character(0L),
+        count = numeric(0L),
+        l_bound = numeric(0L),
+        u_bound = numeric(0L),
+        stringsAsFactors = FALSE
       )
     )
   }
@@ -419,7 +438,8 @@ get_date_range_frequencies <- function(data_object, dataset, variable, extra_par
         level = "1",
         count = length(data_object[[dataset]][[variable]]),
         l_bound = min_val,
-        u_bound = max_val
+        u_bound = max_val,
+        stringsAsFactors = FALSE
       )
     )
   }
@@ -428,8 +448,8 @@ get_date_range_frequencies <- function(data_object, dataset, variable, extra_par
     step <- extra_params$step
   }
   breaks <- seq.Date(min_val, max_val, by = step)
-  if (rev(breaks)[1] != max_val) {
-    breaks[length(breaks) + 1]  <- max_val
+  if (rev(breaks)[1L] != max_val) {
+    breaks[length(breaks) + 1L]  <- max_val
   }
 
   data_object[[dataset]][, variable, drop = FALSE] %>%
@@ -437,27 +457,27 @@ get_date_range_frequencies <- function(data_object, dataset, variable, extra_par
     dplyr::mutate(
       level = factor(
         findInterval(!!sym(variable), breaks, rightmost.closed = FALSE),
-        levels = 1:(length(breaks)),
-        labels = as.character(1:(length(breaks)))
+        levels = seq_along(breaks),
+        labels = as.character(seq_along(breaks))
       )
     ) %>%
     dplyr::group_by(level) %>%
     dplyr::summarise(
       count = dplyr::n()
     ) %>%
-    tidyr::complete(level, fill = list(count = 0)) %>%
+    tidyr::complete(level, fill = list(count = 0L)) %>%
     dplyr::arrange(level) %>%
     dplyr::mutate(
       l_bound = breaks,
-      u_bound = c(breaks[-1], breaks[length(breaks)])
+      u_bound = c(breaks[-1L], breaks[length(breaks)])
     )
 }
 
 #' @rdname filter-source-types
 #' @export
 cb_filter.date_range.tblist <- function(
-  source, type = "date_range", id = .gen_id(), name = id, variable, range = NA,
-  dataset, keep_na = TRUE, ..., description = NULL, active = TRUE) {
+    source, type = "date_range", id = .gen_id(), name = id, variable, range = NA,
+    dataset, keep_na = TRUE, ..., description = NULL, active = TRUE) {
   args <- list(...)
 
   def_filter(
@@ -470,7 +490,7 @@ cb_filter.date_range.tblist <- function(
       if (keep_na && !identical(range, NA)) {
         # keep_na !value_na start
         data_object[[dataset]] <- data_object[[dataset]] %>%
-          dplyr::filter((!!sym(variable) <= !!range[2] & !!sym(variable) >= !!range[1]) | is.na(!!sym(variable)))
+          dplyr::filter((!!sym(variable) <= !!range[2L] & !!sym(variable) >= !!range[1L]) | is.na(!!sym(variable)))
         # keep_na !value_na end
       }
       if (!keep_na && identical(range, NA)) {
@@ -482,7 +502,7 @@ cb_filter.date_range.tblist <- function(
       if (!keep_na && !identical(range, NA)) {
         # !keep_na !value_na start
         data_object[[dataset]] <- data_object[[dataset]] %>%
-          dplyr::filter(!!sym(variable) <= !!range[2] & !!sym(variable) >= !!range[1])
+          dplyr::filter(!!sym(variable) <= !!range[2L] & !!sym(variable) >= !!range[1L])
         # !keep_na !value_na end
       }
       attr(data_object[[dataset]], "filtered") <- TRUE # code include
@@ -501,17 +521,17 @@ cb_filter.date_range.tblist <- function(
         n_data = if ("n_data" %in% name)  data_object[[dataset]][[variable]] %>% stats::na.omit() %>% length(),
         n_missing = if ("n_missing" %in% name) data_object[[dataset]][[variable]] %>% is.na() %>% sum()
       )
-      if (length(name) == 1) {
+      if (length(name) == 1L) {
         return(stats[[name]])
       } else {
         return(stats[name])
       }
     },
-    plot_data = function(data_object) {
+    plot_data = function(data_object, ..., breaks) {
       if (nrow(data_object[[dataset]])) {
-        data_object[[dataset]][[variable]] %>% graphics::hist()
+        data_object[[dataset]][[variable]] %>% graphics::hist(..., breaks = breaks)
       } else {
-        graphics::barplot(0, ylim = c(0, 0.1), main = "No data")
+        graphics::barplot(0.0, ylim = c(0.0, 0.1), main = "No data")
       }
     },
     get_params = function(name) {
@@ -533,8 +553,8 @@ cb_filter.date_range.tblist <- function(
     get_defaults = function(data_object, cache_object) {
       list(
         range = c(
-          cache_object$frequencies$l_bound[1],
-          rev(cache_object$frequencies$u_bound)[1]
+          cache_object$frequencies$l_bound[1L],
+          rev(cache_object$frequencies$u_bound)[1L]
         )
       )
     }
@@ -551,14 +571,150 @@ group_stats <- function(vec_stats, name) {
     stats::setNames(name)
 }
 
+calculate_datetime_step <- function(min_date, max_date) {
+  # Define possible steps
+  steps <- c(
+    "mins" = 60L,
+    "hours" = 3600L,
+    "days" = 86400L,
+    "weeks" = 604800L,
+    "months" = 2592000L,
+    "years" = 31104000L
+  )
+  time_span <- as.numeric(max_date) - as.numeric(min_date)
+  num_elements <- as.integer(time_span / steps)
+  idx <- which(num_elements <= 200L)[1L]
+  if (!is.na(idx)) {
+    return(steps[idx])
+  }
+
+  return(steps[length(steps)])
+}
+
+#' @rdname filter-source-types
+#' @export
+cb_filter.datetime_range.tblist <- function(
+    source, type = "datetime_range", id = .gen_id(), name = id, variable, range = NA,
+    dataset, keep_na = TRUE, ..., description = NULL, active = TRUE) {
+
+  args <- list(...)
+
+  def_filter(
+    type = type,
+    id = id,
+    name = name,
+    input_param = "range",
+    filter_data = function(data_object) {
+      if (keep_na && !identical(range, NA)) {
+        # keep_na !value_na start
+        data_object[[dataset]] <- data_object[[dataset]] %>%
+          dplyr::filter(
+            (!!sym(variable) >= !!range[1L] & !!sym(variable) <= !!range[2L]) |
+              is.na(!!sym(variable))
+          )
+        # keep_na !value_na end
+      }
+      if (!keep_na && identical(range, NA)) {
+        # !keep_na value_na start
+        data_object[[dataset]] <- data_object[[dataset]] %>%
+          dplyr::filter(!is.na(!!sym(variable)))
+        # !keep_na value_na end
+      }
+      if (!keep_na && !identical(range, NA)) {
+        # !keep_na !value_na start
+        data_object[[dataset]] <- data_object[[dataset]] %>%
+          dplyr::filter(
+            !!sym(variable) >= !!range[1L] & !!sym(variable) <= !!range[2L]
+          )
+        # !keep_na !value_na end
+      }
+
+      attr(data_object[[dataset]], "filtered") <- TRUE
+      return(data_object)
+    },
+    get_stats = function(data_object, name) {
+      if (missing(name)) {
+        name <- c("n_data", "frequencies", "n_missing")
+      }
+      extra_params <- list(...)
+
+      data_object[[dataset]][[variable]] <- as.numeric(data_object[[dataset]][[variable]])
+
+      if (is.null(extra_params$step) && !identical(length(data_object[[dataset]][[variable]]), 0L)) {
+        min <- min(data_object[[dataset]][[variable]], na.rm = TRUE)
+        max <- max(data_object[[dataset]][[variable]], na.rm = TRUE)
+
+        extra_params$step <- calculate_datetime_step(min, max) %>% unname()
+      }
+
+      stats <- list(
+        frequencies = if ("frequencies" %in% name) {
+          get_range_frequencies(data_object, dataset, variable, extra_params)
+        },
+        n_data = if ("n_data" %in% name) {
+          data_object[[dataset]][[variable]] %>% stats::na.omit() %>% length()
+        },
+        n_missing = if ("n_missing" %in% name) {
+          data_object[[dataset]][[variable]] %>% is.na() %>% sum()
+        }
+      )
+
+      if (length(name) == 1L) {
+        return(stats[[name]])
+      } else {
+        return(stats[name])
+      }
+    },
+    plot_data = function(data_object, ..., breaks = NULL) {
+      if (nrow(data_object[[dataset]])) {
+        if (is.null(breaks)) {
+          breaks <- calculate_datetime_step(
+            min(data_object[[dataset]][[variable]], na.rm = TRUE),
+            max(data_object[[dataset]][[variable]], na.rm = TRUE)
+          ) %>% names()
+        }
+
+        data_object[[dataset]][[variable]] %>%
+          graphics::hist(..., breaks = breaks)
+      } else {
+        graphics::barplot(0.0, ylim = c(0.0, 0.1), main = "No data")
+      }
+    },
+    get_params = function(name) {
+      params <- list(
+        dataset = dataset,
+        variable = variable,
+        range = range,
+        keep_na = keep_na,
+        description = description,
+        active = active,
+        ...
+      )
+      if (!missing(name)) return(params[[name]])
+      return(params)
+    },
+    get_data = function(data_object) {
+      data_object[[dataset]][[variable]]
+    },
+    get_defaults = function(data_object, cache_object) {
+      list(
+        range = c(
+          cache_object$frequencies$l_bound[1L],
+          rev(cache_object$frequencies$u_bound)[1L]
+        )
+      )
+    }
+  )
+}
+
 #' @rdname filter-source-types
 #' @param variables Vector of variable names to be used in filtering.
 #' @param values Named list of values to be applied in filtering.
 #'   The names should relate to the ones included in `variables` parameter.
 #' @export
 cb_filter.multi_discrete.tblist <- function(
-  source, type = "multi_discrete", id = .gen_id(), name = id, values,
-  variables, dataset, keep_na = TRUE, ..., description = NULL, active = TRUE) {
+    source, type = "multi_discrete", id = .gen_id(), name = id, values,
+    variables, dataset, keep_na = TRUE, ..., description = NULL, active = TRUE) {
   args <- list(...)
 
   def_filter(
@@ -572,7 +728,7 @@ cb_filter.multi_discrete.tblist <- function(
       col_in_val <- function(vec, value, keep_na) {
         if (identical(value, NA)) {
           val_mask <- rep(TRUE, length(vec))
-        } else if (is.null(value)){
+        } else if (is.null(value)) {
           val_mask <- rep(FALSE, length(vec))
         } else {
           val_mask <- vec %in% value
@@ -586,9 +742,11 @@ cb_filter.multi_discrete.tblist <- function(
 
       data_object[[dataset]] <- data_object[[dataset]] %>%
         dplyr::filter(
-          dplyr::across(
-            !!names(values),
-            ~col_in_val(.x, values[[dplyr::cur_column()]], !!keep_na)
+          dplyr::if_all(
+            dplyr::all_of(names(values)),
+            # Using deparse(substitute(.x)) over dplyr::cur_column
+            # dplyr::cur_column is accessible only in across
+            ~ col_in_val(.x, values[[deparse(substitute(.x))]], !!keep_na)
           )
         )
       attr(data_object[[dataset]], "filtered") <- TRUE
@@ -605,21 +763,22 @@ cb_filter.multi_discrete.tblist <- function(
         n_data = if ("n_data" %in% name)  data_object[[dataset]][variables] %>% nrow(),
         n_missing = if ("n_missing" %in% name) data_object[[dataset]][variables] %>% is.na() %>% colSums() %>% as.list()
       )
-      if (length(name) == 1) {
+      if (length(name) == 1L) {
         return(stats[[name]])
       } else {
         return(stats[name])
       }
     },
-    plot_data = function(data_object) {
+    plot_data = function(data_object, ...) {
+      variables <- unlist(variables)
       if (nrow(data_object[[dataset]])) {
-        data_object[[dataset]][names(variables)] %>%
+        data_object[[dataset]][variables] %>%
           purrr::map(table) %>%
           purrr::imap_dfc(group_stats) %>%
           as.matrix() %>%
-          graphics::barplot()
+          graphics::barplot(...)
       } else {
-        graphics::barplot(0, ylim = c(0, 0.1), main = "No data")
+        graphics::barplot(0.0, ylim = c(0.0, 0.1), main = "No data")
       }
     },
     get_params = function(name) {
@@ -652,8 +811,8 @@ cb_filter.multi_discrete.tblist <- function(
 #' @param keep_na If `TRUE`, NA values are included.
 #' @export
 cb_filter.query.tblist <- function(
-  source, type = "query", id = .gen_id(), name = id, variables, value = NA,
-  dataset, keep_na = TRUE, ..., description = NULL, active = TRUE) {
+    source, type = "query", id = .gen_id(), name = id, variables, value = NA,
+    dataset, keep_na = TRUE, ..., description = NULL, active = TRUE) {
   args <- list(...)
 
   def_filter(
@@ -688,21 +847,21 @@ cb_filter.query.tblist <- function(
         n_data = if ("n_data" %in% name)  data_object[[dataset]][variables] %>% nrow(),
         n_missing = if ("n_missing" %in% name) data_object[[dataset]][variables] %>% is.na() %>% colSums() %>% as.list()
       )
-      if (length(name) == 1) {
+      if (length(name) == 1L) {
         return(stats[[name]])
       } else {
         return(stats[name])
       }
     },
-    plot_data = function(data_object) {
+    plot_data = function(data_object, ...) {
       if (nrow(data_object[[dataset]])) {
         data_object[[dataset]][variables] %>%
           purrr::map(table) %>%
           purrr::imap_dfc(group_stats) %>%
           as.matrix() %>%
-          graphics::barplot()
+          graphics::barplot(...)
       } else {
-        graphics::barplot(0, ylim = c(0, 0.1), main = "No data")
+        graphics::barplot(0.0, ylim = c(0.0, 0.1), main = "No data")
       }
     },
     get_params = function(name) {
@@ -740,10 +899,10 @@ cb_filter.query.tblist <- function(
   }
 
   key_values <- NULL
-  common_key_names <- paste0("key_", seq_along(binding_key$data_keys[[1]]$key))
+  common_key_names <- paste0("key_", seq_along(binding_key$data_keys[[1L]]$key))
   for (dependent_dataset in dependent_datasets) {
     key_names <- binding_key$data_keys[[dependent_dataset]]$key
-    tmp_key_values <- dplyr::distinct(data_object_post[[dependent_dataset]][, key_names, drop = FALSE]) %>%
+    tmp_key_values <- collapse::funique(data_object_post[[dependent_dataset]][, key_names, drop = FALSE]) %>%
       stats::setNames(common_key_names)
     if (is.null(key_values)) {
       key_values <- tmp_key_values
@@ -752,15 +911,28 @@ cb_filter.query.tblist <- function(
     }
   }
 
-  data_object_post[[binding_dataset]] <- dplyr::inner_join(
-    switch(
-      as.character(binding_key$post),
-      "FALSE" = data_object_pre[[binding_dataset]],
-      "TRUE" = data_object_post[[binding_dataset]]
-    ),
-    key_values,
-    by = stats::setNames(common_key_names, binding_key$update$key)
+  df <- switch(
+    as.character(binding_key$post),
+    "FALSE" = data_object_pre[[binding_dataset]],
+    "TRUE" = data_object_post[[binding_dataset]]
   )
+
+  data_object_post[[binding_dataset]] <- tryCatch({
+    collapse::join(
+      df,
+      key_values,
+      on = stats::setNames(common_key_names, binding_key$update$key),
+      how = "inner",
+      verbose = getOption("cb_verbose", default = FALSE)
+    )
+  }, error = function(e) {
+    dplyr::inner_join(
+      df,
+      key_values,
+      by = stats::setNames(common_key_names, binding_key$update$key)
+    )
+  })
+
   if (binding_key$activate) {
     attr(data_object_post[[binding_dataset]], "filtered") <- TRUE
   }
@@ -781,7 +953,7 @@ cb_filter.query.tblist <- function(
     if (is.null(pkey)) {
       return(dataset)
     } else {
-      dataset_pkey <- .get_item(pkey, "dataset", dataset)[1][[1]]$key
+      dataset_pkey <- .get_item(pkey, "dataset", dataset)[1L][[1L]]$key
       if (is.null(dataset_pkey)) return(dataset)
       return(glue::glue("{dataset}\n primary key: {paste(dataset_pkey, collapse = ', ')}"))
     }
@@ -800,8 +972,8 @@ cb_filter.query.tblist <- function(
     ) %>%
       purrr::map(~names(.[["data_keys"]])) %>%
       unlist() %>%
-      unique()
-    if (length(dependent_datasets) > 0) {
+      collapse::funique()
+    if (length(dependent_datasets) > 0L) {
       bind_keys_section <- glue::glue(
         "\nData linked with external datasets: {paste(dependent_datasets, collapse = ', ')}",
         .trim = FALSE
